//*****************************************************************************
//*****************************************************************************
//  FILENAME: DelSig.h
//   Version: 1.1, Updated on 2006/06/12 at 14:52:34
//
//  DESCRIPTION:  C declarations for the DelSig User Module with a 2nd-order
//                modulator based on a type-2 dcecimator
//-----------------------------------------------------------------------------
//      Copyright (c) Cypress MicroSystems 2004. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************

#include <m8c.h>

#pragma fastcall16 DelSig_Start
#pragma fastcall16 DelSig_SetPower
#pragma fastcall16 DelSig_StartAD
#pragma fastcall16 DelSig_StopAD
#pragma fastcall16 DelSig_Stop
#pragma fastcall16 DelSig_fIsDataAvailable
#pragma fastcall16 DelSig_iGetData
#pragma fastcall16 DelSig_wGetData
#pragma fastcall16 DelSig_iGetDataClearFlag
#pragma fastcall16 DelSig_wGetDataClearFlag
#pragma fastcall16 DelSig_ClearFlag
#pragma fastcall16 DelSig_WritePulseWidth

//-------------------------------------------------
// Prototypes of the DelSig API.
//-------------------------------------------------
extern void DelSig_Start(BYTE bPower);
extern void DelSig_SetPower(BYTE bPower);
extern void DelSig_StartAD(void);
extern void DelSig_StopAD(void);
extern void DelSig_Stop(void);
extern BOOL DelSig_fIsDataAvailable(void);
extern INT  DelSig_iGetData(void);
extern WORD DelSig_wGetData(void);
extern INT  DelSig_iGetDataClearFlag(void);
extern WORD DelSig_wGetDataClearFlag(void);
extern void DelSig_ClearFlag(void);
extern void DelSig_WritePulseWidth(WORD wPulseWidthValue);

//-------------------------------------------------
// Defines for DelSig API's.
//-------------------------------------------------
#define DelSig_OFF                         (0)
#define DelSig_LOWPOWER                    (1)
#define DelSig_MEDPOWER                    (2)
#define DelSig_HIGHPOWER                   (3)

#define DelSig_DATA_READY_BIT              (0x10)
#define DelSig_2S_COMPLEMENT               (0)

//-------------------------------------------------
// Hardware Register Definitions
//-------------------------------------------------

#pragma ioport  DelSig_PWM_DR0_MSB: 0x024           //Time base Counter register
BYTE            DelSig_PWM_DR0_MSB;
#pragma ioport  DelSig_PWM_DR0_LSB: 0x020
BYTE            DelSig_PWM_DR0_LSB;
#pragma ioport  DelSig_PWM_DR1_MSB: 0x025           //Time base Period value register
BYTE            DelSig_PWM_DR1_MSB;
#pragma ioport  DelSig_PWM_DR1_LSB: 0x021
BYTE            DelSig_PWM_DR1_LSB;
#pragma ioport  DelSig_PWM_DR2_MSB: 0x026           //Time base CompareValue register
BYTE            DelSig_PWM_DR2_MSB;
#pragma ioport  DelSig_PWM_DR2_LSB: 0x022
BYTE            DelSig_PWM_DR2_LSB;
#pragma ioport  DelSig_PWM_CR0_MSB: 0x027           //Time base Control register
BYTE            DelSig_PWM_CR0_MSB;
#pragma ioport  DelSig_PWM_CR0_LSB: 0x023
BYTE            DelSig_PWM_CR0_LSB;
#pragma ioport  DelSig_PWM_FN_MSB:  0x124            //Time base Function register
BYTE            DelSig_PWM_FN_MSB;
#pragma ioport  DelSig_PWM_FN_LSB:  0x120
BYTE            DelSig_PWM_FN_LSB;
#pragma ioport  DelSig_PWM_SL_MSB:  0x125            //Time base Input register
BYTE            DelSig_PWM_SL_MSB;
#pragma ioport  DelSig_PWM_SL_LSB:  0x121
BYTE            DelSig_PWM_SL_LSB;
#pragma ioport  DelSig_PWM_OS_MSB:  0x126            //Time base Output register
BYTE            DelSig_PWM_OS_MSB;
#pragma ioport  DelSig_PWM_OS_LSB:  0x122
BYTE            DelSig_PWM_OS_LSB;

#pragma ioport  DelSig_AtoD1cr0:    0x080              //Analog block 1 control register 0
BYTE            DelSig_AtoD1cr0;
#pragma ioport  DelSig_AtoD1cr1:    0x081              //Analog block 1 control register 1
BYTE            DelSig_AtoD1cr1;
#pragma ioport  DelSig_AtoD1cr2:    0x082              //Analog block 1 control register 2
BYTE            DelSig_AtoD1cr2;
#pragma ioport  DelSig_AtoD1cr3:    0x083              //Analog block 1 control register 3
BYTE            DelSig_AtoD1cr3;
#pragma ioport  DelSig_AtoD2cr0:    0x090              //Analog block 2 control register 0
BYTE            DelSig_AtoD2cr0;
#pragma ioport  DelSig_AtoD2cr1:    0x091              //Analog block 2 control register 1
BYTE            DelSig_AtoD2cr1;
#pragma ioport  DelSig_AtoD2cr2:    0x092              //Analog block 2 control register 2
BYTE            DelSig_AtoD2cr2;
#pragma ioport  DelSig_AtoD2cr3:    0x093              //Analog block 2 control register 3
BYTE            DelSig_AtoD2cr3;


// end of file DelSig.h
