//_________________________________________________________________________________-_______________________
// OakLibTest_noHotplug.cpp
// 
// This is a very simple console application that demonstrates how to communicate with the Oak sensors,
// using the Oak library provided by Toradex.
// 
// It was tested in Visual Studio 2005 running on Windows XP
//
// June 29, 2007, Andy Kiser, Toradex AG
//________________________________________________________________________________________________________
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include "Oak.h"

#ifdef _UNICODE
#error ASCII mode required: check project properties -> Configuration Properties -> General -> Character Set
#endif

#define MAX_REPORTBYTES		33
#define CALPOS_RAWVALUE		840
#define CALPOS_RAWVALUE_MAX	1080
#define CALPOS_RAWVALUE_MIN	600
#define CALPOS_MEASTOL		40

#define ANZMEASURES			20

// __________________________ _t m a i n _________________________________________________________________
// this is already the main function.
// As you see, I tried to keep the complexity of the application low, so you can easily find what this 
// application wants to show: communication with the Oak sensor.

int _tmain(int argc, _TCHAR* argv[])
{
	bool		fSuccess;
	tOakSensor	Sensor;							// details about the Oak sensor and its data channels (name, data format, ...)
	BYTE		Report[MAX_REPORTBYTES];		// USB data transferred in reports.
	
	BYTE		bReportSize;

	unsigned short usRawValues_Sum;
	unsigned short usRawValue;
	unsigned short usRawValue_n;
	short sOffset;	

	INT i;
	unsigned short usValue;
	char ch;

	printf_s("\n===============================================\n");
    printf_s("Sensor ""Oak Dist"" V 2.x Calibration Program    \n");
	printf_s("===============================================\n\n");

	// initialization: Find one specific sensor
	// Note: this is a basic application that only reacts on one Oak Sensor, even if multiple are connected

	fSuccess = Oak_FindSensor( 0x0005,	// search PID for Oak Dist (product ID)
							   0x0000,	// search for ANY REV (product revision)
							   NULL,	// search for ANY serial number
							   NULL,	// search for ANY device name
							   NULL,	// search for ANY channel name
							   NULL,	// search for ANY user defined device name
							   NULL,	// search for ANY user defined channel name
							   &Sensor); // <- store inforation about the first Oak sensor that matches 
										 //    the criteria above into this variable

	if (!fSuccess)	// no matching sensor found --> just leave
	{
		printf_s("/* ERROR: No ""Oak Dist"" Sensor found */\n"); // this should never happen
		goto LableError;
	}

	printf_s("Sensor found VID:%04X  PID:%04X  REV:%04X\n", Sensor.VID, Sensor.PID, Sensor.REV);

	printf_s("Calbration runs .....\n\n", Sensor.VID, Sensor.PID, Sensor.REV);

	//------ calulate some sensor depend parameter ------------------------------------------------
	bReportSize = 0;
	for (i=0; i<(Sensor.NumChannels); i++)
	{
		bReportSize = bReportSize+(Sensor.Channel[i].BitSize/8);	// calculate Anz of bytes
	}
	bReportSize++;			// one byte more see report description
	
	if (MAX_REPORTBYTES < bReportSize)
	{
		printf_s("/* ERROR: Report Size to high */\n"); // this should never happen
		goto LableError;
	}

	//-------- set sample rate --------------------------------------------------------------------
	//set report to the device
	Report[0]= 0;		// allways 0 
	Report[1]= 0;		// Out report 
	Report[2]= 0;		// Set in RAM
	Report[3]= 0x02;	// Reportsize 1 byte
	Report[4]= 0x01;	// 2th 8Bit paramater sensor type 
	Report[5]= 0x00;	
	Report[6]= 0x06;	// no sensor RAW values
	Report[7]= 0x00;

	fSuccess = Oak_Feature(Sensor.DevicePath, Report, true);
	if (!fSuccess)
	{
		printf_s("\n/* ERROR: Feature Report ""set sample rate"" failed */\n");
		goto LableError;
	}

	//-------- set Sensor type to RAW mode -----------------------------------------------------------
	//set report to the device
	Report[0]= 0;		// allways 0 
	Report[1]= 0;		// Out report 
	Report[2]= 0;		// Set in RAM
	Report[3]= 0x01;	// Reportsize 1 byte
	Report[4]= 0x02;	// 2th 8Bit paramater sensor type 
	Report[5]= 0x00;	
	Report[6]= 0xFF;	// no sensor RAW values


	fSuccess = Oak_Feature(Sensor.DevicePath, Report, true);
	if (!fSuccess)
	{
		printf_s("\n/* ERROR: Feature Report ""set sensor type (raw values)"" failed */\n");
		goto LableError;
	}


	//-------- get sensor type --------------------------------------------------------------------
	//set report to the device
	Report[0]= 0;		// allways 0 
	Report[1]= 1;		// In report 
	Report[2]= 0;		// Set in RAM
	Report[3]= 0x01;	// Reportsize 1 byte
	Report[4]= 0x02;	// 2th 8Bit paramater sensor type 
	Report[5]= 0x00;	

	fSuccess = Oak_Feature(Sensor.DevicePath, Report, true);
	if (!fSuccess)
	{
		printf_s("\n/* ERROR: Feature Report ""get sensor type"" failed */\n");
		return -1;
	}

	if (0xFF != Report[2])
	{
		printf_s("\n/* ERROR: Set ""sensor type"" failed */\n");
		goto LableError;
	}

	// wait until the sensor measure raw valus (depend on the orginal samle rate of 500ms)
	Sleep(700);

	//------- some security dummy reads to get all "old" values out of the buffers ----------------------
	for (i=0; i<5; i++)
	{
		fSuccess = Oak_GetCurrentInReport(Sensor.DevicePath, Report,bReportSize);
		if (!fSuccess)
		{
			printf_s("/* ERROR: CurrentInReport failed */\n"); // this should never happen
			goto LableError;
		}
	}	

	//------- read the calibration posiion as a raw value -------------------------------------------------
	usRawValues_Sum = 0;
	printf_s("\n");
	for (i=0; i<ANZMEASURES; i++)
	{
		fSuccess = Oak_GetCurrentInReport(Sensor.DevicePath, Report,bReportSize);
		if (!fSuccess)
		{
			printf_s("/* ERROR: CurrentInReport failed */\n"); // this should never happen
			goto LableError;
		}
		usRawValue = (unsigned short)Report[3]+(unsigned short)(Report[4]*256);  // calulate the distanze

		printf_s("%d,",usRawValue); // this should never happen

		if (usRawValue > CALPOS_RAWVALUE_MAX)
		{
			printf_s("/* ERROR: Divergence, the Raw Value is to high %d */\n",usRawValue); // this should never happen
			goto LableError;
		}

		if (usRawValue < CALPOS_RAWVALUE_MIN) // no negativ tolerance	
		{
			printf_s("/* ERROR: Divergence, the Raw Value is to low */\n"); // this should never happen
			goto LableError;
		}

		if (i>1)		// at least on measurement was happen usRawValue_n is defined 
		{
			sOffset = (short)usRawValue_n-(short)usRawValue;					// difference 
			if (abs(sOffset) > CALPOS_MEASTOL)
			{
				printf_s("/* ERROR: Unstable, the jitter of the Raw Value is to big */\n"); // this should never happen
				goto LableError;
			}
		}
		usRawValue_n=usRawValue;						// save the current value fo the next test n+1
		
		usRawValues_Sum = usRawValues_Sum+usRawValue;	// caculate the total
		Sleep(20);
	}	

	usRawValue = usRawValues_Sum/ANZMEASURES;

	// calculate the offset
	sOffset = CALPOS_RAWVALUE-(short)usRawValue;		// difference 

	printf_s("Offset = %d\n",sOffset);

	//-------- set the offset ----------------------------------------------------------------------
	//set report to the device
	Report[0]= 0;		// allways 0 
	Report[1]= 0;		// Out report 
	Report[2]= 1;		// Set in FLASH
	Report[3]= 0x02;	// Reportsize 2 byte
	Report[4]= 0x03;	// 3th 16Biit paramater sensor type 
	Report[5]= 0x00;	
	Report[6]= (unsigned short)sOffset&0xFF;	   // set low byte
	Report[7]= ((unsigned short)sOffset>>8)&0xFF;  // set high byte

	fSuccess = Oak_Feature(Sensor.DevicePath, Report, true);
	if (!fSuccess)
	{
		printf_s("\n/* ERROR: Set teh Offset failed */\n");
		goto LableError;
	}

	//-------- set to normal mode -----------------------------------------------------------------
	//set report to the device
	Report[0]= 0;		// allways 0 
	Report[1]= 0;		// Out report 
	Report[2]= 0;		// Set in RAM
	Report[3]= 0x01;	// Reportsize 1 byte
	Report[4]= 0x02;	// 2th 8Bit paramater sensor type 
	Report[5]= 0x00;	
	Report[6]= 0x00;	// with sensor normal mode

	fSuccess = Oak_Feature(Sensor.DevicePath, Report, true);
	if (!fSuccess)
	{
		printf_s("\n/* ERROR: Set to ""Read values"" failed */\n");
		goto LableError;
	}


	//--------- additinal test (only internaly used -----------------------------------------------
#if 0
	char ch;
	FILE * pFile;
	unsigned short usTest[80];
	char str [80];

	printf ("\nScan the sensor: ");
	scanf ("%s",str);
	ch = getchar();

	for (i=10; i<=80; i+=10)
	{
		printf_s("Set the senor to position %d cm !",i);
		ch = getchar();
		fSuccess = Oak_GetCurrentInReport(Sensor.DevicePath, Report,bReportSize);
		if (!fSuccess)
		{
			printf_s("/* ERROR: GetReport failed */\n"); // this should never happen
			return -1;
		}
		usValue = (unsigned short)Report[3]+(unsigned short)(Report[4]*256);  // calulate the distanze
		
		usTest[i]=usValue;
	}

	pFile = fopen ("Results.txt","a");

	if (pFile)	
	{
		fprintf_s(pFile,"\nx");
		for (i=10; i<=80; i+=10)
		{
			fprintf_s(pFile,";%d",usTest[i]);
		}
		fprintf_s(pFile,";%s\n",str);
		fclose(pFile);
	}
#endif
	printf_s("\nCalbration sucessful finished\n");

	printf_s("press Return to continue ..");
	ch = getchar();
	return 0;
	
	//-------- cas of error --------------------------------------------------------------------------
LableError:
	printf_s("press Return to continue ..");
	ch = getchar();
	return -1;
}

